function [tSurface] = gp3173_gSurf (ecR, sR, f, sGsurfN, teCurves, bxdim, budim, ndig)

% RETURNS:
% tSurface structure with all NUBRS features of surface, un-trimmed, i.e.
% with 4 contour curves


% INPUTS:
% ecR: edge curves references for that surface
% sR: surface ref. in sGarrays: sGsurf
% sGsurfN: surface features from sGarrays
% teCurves: gShell curves computed previously
% bxdim, budim: dimensions of surface

% There are 3 cases:

% A.- Non-planar surface with no-linear directions: copy directly from sG arrays

% B.- Planar surface with both directions non-linear: 'fills' the interior
% with specific algorithm

% C.- Surfaces with at least 1 linear direction: just re-arranges cp's
% using the pair of non-linear curves



% 1.- DECLARATION OF STRUCTURE and EXTRACTION OF sGsurf NURBS FEATURES:
        
tSurface = struct ('Ncp',[], 'Degree',[], 'Usize',[], 'KnotV',[], 'w',[], 'P',[], 'PL',1, 'xdim', 0, 'udim', 0, 'CS',[]);
        
sgNcp    = sGsurfN(sR).Ncp;
sgDegree = sGsurfN(sR).Degree ;
sgUsize  = sGsurfN(sR).Usize ;
        
sgKnotV  = sGsurfN(sR).KnotV ;
sgw      = sGsurfN(sR).w ;
sgP      = sGsurfN(sR).P ;
sgCS     = sGsurfN(sR).OC ;
        
% tSurface dimensions:
tSurface.xdim = bxdim ;
tSurface.udim = budim ;
        
% t Surface closed flag:
tSurface.CS = sgCS ;
        
% determines if all P are in the same plane:
tSurface.PL = sf3151_surfacePL (sgNcp, sgP);


% 2.- TRANSFERS FEATURES FROM EDGE CURVES (EXCEPT CP'S COORD'S):
        
for di = 1:1:2
               
    cR = ecR(di,1);

    tSurface.Ncp(di)     = teCurves(cR).Ncp;
    tSurface.Degree(di)  = teCurves(cR).Degree;
    tSurface.Usize(di)   = teCurves(cR).Usize;

    for i = 1:1:teCurves(cR).Usize
        tSurface.KnotV(di,i) = round(teCurves(cR).KnotV(i),6);
    end             
end
        

% 3.- CONTROL POINTS CALCULATION:

% 3.1.- CASE A
% Non-planar surfaces and both directions non-linear
% transfers info from sGarrays, but afterwards needs to homogenize 
% Ncp, Degree and KnotV

% arranges features to match global orientation. To know orientation the 
% surface must have uses its curves (allocated previously to that face)
        
if tSurface.PL == 0 && ...
(teCurves(ecR(1,1)).Ncp > 2 && teCurves(ecR(2,1)).Ncp > 2)


	% A.1- Extracts 1st and 2nd cp's coordinates: for current 
	% face and for orientation that has to be

	% cO(1) current origin of the surface (directly from sGsurfaces):
 	% cO(2) second cp of face line 1,1 (direction 1 from cO(1))
    % i.e line cO points from origin to direction 1
    
    sO  = zeros(bxdim,1); % first cp position
    vs1 = zeros(bxdim,1); % vector at origin in direction 1
    
    for di = 1:1:bxdim
        
        sO(di)  = sgP(1,di);
        vs1(di) = sgP(sgNcp(2)+1,di) - sgP(1,di);
        
        %cO(1,di) = sgP(1,di);
     	%cO(2,di) = sgP(sgNcp(2)+1,di);
    end
 
    vs1 = vs1/norm(vs1);
    
      
	% A.2.- detects what edge curve cO lies in and what orientation:
   	% stores in 'O' first and second point of the curve
  	% stores in cC(3,1) coincident curve indexes and sense (1 or -1).
    % Compares curve initial vector with surface initial vector (in direction 1)
            

 	cC = zeros(1,3); % 'coincident curve and sense w.r.t. curve that has to be
           
    for i = 1:1:2
	for j = 1:1:2
              
        cR = ecR(i,j);

        % positive sense
        
        vc = zeros(bxdim,1);
        cO = zeros(bxdim,1);
        
        for di = 1:1:bxdim
            vc(di) = teCurves(cR).P(2,di)-teCurves(cR).P(1,di);
            cO(di) = teCurves(cR).P(1,di);
        end
        vc = vc/norm(vc);
        
        if go2542_coincidentP(vs1,vc,bxdim,2) == 1 && go2542_coincidentP(sO,cO,bxdim,2) == 1
            cC = [i j 1];
        end
        

        % negative sense:
            
     	cNcp = teCurves(cR).Ncp;
        
        for di=1:1:bxdim
            vc(di) = teCurves(cR).P(cNcp-1,di)-teCurves(cR).P(cNcp,di);
            cO(di) = teCurves(cR).P(cNcp,di);
        end
        vc = vc/norm(vc);
        
        if go2542_coincidentP(vs1,vc,bxdim,2) == 1 && go2542_coincidentP(sO,cO,bxdim,2) == 1
            cC = [i j -1];
        end
        

        
        %{
        if f==2 && i==2 && j==2
            figure(44)
            
            [i j]
            
            for g = 1:1: teCurves(cR).Ncp
            
                scatter3( teCurves(cR).P(g,1),  teCurves(cR).P(g,2),  teCurves(cR).P(g,3), 'r'); hold on
                text( teCurves(cR).P(g,1),  teCurves(cR).P(g,2),  teCurves(cR).P(g,3), num2str(g)); hold on
            end
            axis equal
            
        end
        %}
        
	end
    end
         


    
	% A.3.- changes sGsurfaces NURBS features, acc/to curve 
  	% coincidence and orientation, and allocates:
            
	% copy from sGsurfaces
  	[tSurface] = sf3159_Copy ...
   	(tSurface, sgNcp, sgDegree, sgUsize, sgKnotV, sgw, sgP);
            
 	% case Aa.1
    if (cC(1) == 1 && cC(2) == 1) && cC(3)==1
            
        % stats
                
 	% case Aa.2
	elseif (cC(1) == 1 && cC(2) == 1) && cC(3)==-1
        
        [tSurface] = sf3152_ReverseCPs (tSurface, 1);
                
	% case Ab.1
   	elseif (cC(1) == 1 && cC(2) == 2) && cC(3)== 1
                
        [tSurface] = sf3152_ReverseCPs (tSurface, 2);
                
 	% case Ab.2
  	elseif (cC(1) == 1 && cC(2) == 2) && cC(3)==-1
                
        [tSurface] = sf3152_ReverseCPs (tSurface, 1);
        [tSurface] = sf3152_ReverseCPs (tSurface, 2);
                
 	% case Ac.1
	elseif (cC(1) == 2 && cC(2) == 1) && cC(3)== 1
            
        [tSurface] = sf3153_Swap (tSurface);
                
	% case Ac.2
	elseif (cC(1) == 2 && cC(2) == 1) && cC(3)==-1
                
        [tSurface] = sf3152_ReverseCPs (tSurface, 1);
        [tSurface] = sf3153_Swap (tSurface);
                
    % case Ad.1
	elseif (cC(1) == 2 && cC(2) == 2) && cC(3)== 1

        [tSurface] = sf3152_ReverseCPs (tSurface, 2);
        [tSurface] = sf3153_Swap (tSurface);
                  
	% case Ad.2
 	elseif (cC(1) == 2 && cC(2) == 2) && cC(3)==-1
                
        [tSurface] = sf3152_ReverseCPs (tSurface, 1);
        [tSurface] = sf3152_ReverseCPs (tSurface, 2);
    	[tSurface] = sf3153_Swap (tSurface);
                
    end


    % A.4.- HOMOGENIZES DEGREE, NCP AND KNOTV:
    
    [tSurface] = gp3175_surfHomog (tSurface, teCurves, ecR, ndig);
    
    
% 3.2.- CASE B
% Planar surfaces with both directions non-linear,
% fills the surface: uses contour curves NURBS features
        
elseif (tSurface.PL == 1) && ...
(teCurves(ecR(1,1)).Ncp > 2 && teCurves(ecR(2,1)).Ncp > 2)
         

    % for checking parametrization metrics:
    
    Nsp = 5 ; % No. sample points in each direction
    
    ULims = [   tSurface.KnotV(1,1) tSurface.KnotV(1,tSurface.Usize(1)); ...
                tSurface.KnotV(2,1) tSurface.KnotV(2,tSurface.Usize(2))];
    
    ULims = round(ULims,4);
            
	Ust = zeros(budim,1);
    
    for di = 1:1:budim
        Ust(di) = (ULims(di,2) - ULims(di,1))/(Nsp-1);
    end
    
    
    
	% B.1.- transfers to eP() and ew() all edges curves points coordinates and weights:
            
	eP = zeros(tSurface.Ncp(1), tSurface.Ncp(2), bxdim); 
	ew = zeros(tSurface.Ncp(1), tSurface.Ncp(2)); 
            
    for di=1:1:bxdim
                
        jend = tSurface.Ncp(1);
                
        for j = 1:1:tSurface.Ncp(2)
                    
            eP(1,j,di)      = teCurves(ecR(2,1)).P(j,di);
            eP(jend,j,di)   = teCurves(ecR(2,2)).P(j,di);
                    
            ew(1,j)     	= teCurves(ecR(2,1)).w(j);
            ew(jend,j)     	= teCurves(ecR(2,2)).w(j);
                    
        end

                
        iend = tSurface.Ncp(2);
                
        for i = 2:1:tSurface.Ncp(1)-1
                    
            eP(i,1,di)      = teCurves(ecR(1,1)).P(i,di);
            eP(i,iend,di)   = teCurves(ecR(1,2)).P(i,di);
                    
          	ew(i,1)     	= teCurves(ecR(1,1)).w(i);
         	ew(i,iend)     	= teCurves(ecR(1,2)).w(i);
                    
        end      
    end
        

    
    % B.2.- Fills inside contour by deformation of envelope rectangle:
    
    Ncp = tSurface.Ncp;
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %{
    if sR==3
        tm = zeros(1,3);
        conta = 0;
        for i =1:1:Ncp(1)
        for j =1:1:Ncp(2)

            conta  = conta+1;

            tm(conta,:) = eP(i,j,:);

        end
        end

        figure(1)
        scatter3(tm(:,1),tm(:,2),tm(:,3),'r'); hold on
        axis equal
    end
    %}
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    

    
	% Rotates eP to match horizontal plane:
 	[eaPi, Pr, vr, hztF, tita] = sf3154_AbateP (eP);

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%{
    
    if sR==6
        
        sc = 500 ;
        
        tm=zeros(1,3);
        conta = 0;
        for i =1:1:Ncp(1)
        for j =1:1:Ncp(2)

            conta  = conta+1;

            tm(conta,:) = eaPi(i,j,:);

        end
        end

        figure(2)
        scatter3(tm(:,1),tm(:,2),tm(:,3),'r'); hold on
        axis equal
        
        scatter3(Pr(1), Pr(2), Pr(3), 'b'); hold on

        
        plot3( [Pr(1),Pr(1)+sc*vr(1)] , [Pr(2),Pr(2)+sc*vr(2)] ,[Pr(3),Pr(3)+sc*vr(3)] ,'b'); hold on
        
        if gg==0
        end
        
    end
    
	%}
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
    
    
    
    
    % lattice fitting with parametric optimization:
    [fP] = sf3156_plLattFitt (Ncp, eaPi, sR);
    
    
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %{
    if sR==6
        fP
        
        figure(sR)
        for i = 1:1:3

            scatter3(fP(i,:,1), fP(i,:,2), fP(i,:,3), 'b'); hold on

        end

        axis equal
        
        if gg==8
        end
        
    end
    %}
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    
    % returns plane to its position:
    if hztF == 0
        
        [fP] = sf3155_AbatePback (fP, Pr, vr, tita);
        
    end


    
    
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %{
        if sR == 5

            figure(4)
            
            for i = 1:1:size(fP,1)
                %scatter3(fPS(i,:,1),fPS(i,:,2),fPS(i,:,3),'r'); hold on
                plot3(fP(i,:,1),fP(i,:,2),fP(i,:,3),'-r'); hold on
            end

            for j = 1:1:size(fP,2)
                plot3(fP(:,j,1),fP(:,j,2),fP(:,j,3),'-r'); hold on
            end
            axis equal
        end
        %}
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        
    
	% B.3.- allocates cp's coordinates to tSurface:
	% (internal weights are assumed to be one)
            
	conta = 0;
            
 	for i = 1:1:tSurface.Ncp(1)
        for j = 1:1:tSurface.Ncp(2)
                
            conta = conta+1;
                
            for di=1:1:bxdim
                tSurface.P(conta,di) = fP(i,j,di);
            end
                
            if (i==1 || i==tSurface.Ncp(1)) || (j==1 || j==tSurface.Ncp(2))
                tSurface.w(conta) = ew(i,j);
            else
                tSurface.w(conta) = 1 ;
            end
                
        end
	end
        
            
% 3.3.- CASE C
% Rest of the cases: planar / non-planar surfaces with AT LEAST ONE LINEAR
% direction, merely allocates edge curves cp's coordinates RE-ARRANGED: sandwich algorithm

else
     
    conta = 0 ;
           
    for po = 1:1:tSurface.Ncp(2)
               
        conta = conta + 1 ;
        for di=1:1:bxdim
        	tSurface.P(conta,di) = teCurves(ecR(2,1)).P(po,di);
        end
        tSurface.w(conta) = teCurves(ecR(2,1)).w(po);
    end
           
    for po = 2:1:tSurface.Ncp(1)-1
        
        for j = 1:1:2
                   
            conta = conta + 1 ;
            for di=1:1:bxdim
                tSurface.P(conta,di) = teCurves(ecR(1,j)).P(po,di);
            end
            tSurface.w(conta) = teCurves(ecR(1,j)).w(po);
        end
    end
           
    for po = 1:1:tSurface.Ncp(2)
               
        conta = conta + 1 ;
      	for di=1:1:bxdim
        	tSurface.P(conta,di) = teCurves(ecR(2,2)).P(po,di);
        end
        tSurface.w(conta) = teCurves(ecR(2,2)).w(po);
    end
    
end 
        
        
        
% makes sure knot vector start / end components are 0 and 1 
% (can vary slightly due to rouding up during calculations !)

tsp = tSurface.Degree;
tsu = tSurface.Usize;
        
for di = 1:1:2       
    for i=1:1:tsp(di)+1
        tSurface.KnotV(di,i) = 0 ;
    end

    for j = tsu(di)-tsp(di):1:tsu(di)
        tSurface.KnotV(di,j) = 1 ;
    end        
end

        
end

